#!/bin/bash

LTE_RST_CHIP=
LTE_RST_PIN=
LTE_LED=
DEV=cdc-wdm0
IFNAME=wwan0
CONNEECTION_NAME=wwan-lte-001
I2C_BUS=10
I2C_ADD=20
PING_IP_1="223.5.5.5"
PING_IP_2="8.8.8.8"
is_ipc=1

BSP_HOME_PATH=$(dirname $(realpath $0))

BASE_CONFIG_INI=/usr/share/ed-lte-qmi/ed_config.ini
read_ini_file(){
    if [ ! -f $1 ];then
        return 1
    fi
    v=$(awk -F '=' '/\['$2'\]/{a=1}a==1&&$1~/'$3'/{print $2;exit}' $1)
    echo $v
    if [ ${#src} -eq 0 ];then
        return 1
    fi
    return 0
}

# 是否是NetworkManager 管理
function is_networkmanager(){
    local t_net=`ps -ef | grep NetworkManager | grep -v grep | wc -l`
    if [ $t_net -eq 0 ];then
        return 1
    fi
    return 0
}

function echo_log(){
    local time_now=$(date +%Y/%m/%d,%H:%M:%S)
    echo "[$time_now] $1"
}

function die(){
    echo_log "ERROR: $1"
    exit 1  
}

function do_lte_init_rst_pin() {
    if [ $is_ipc -ne 1 ];then
        LTE_RST_PIN=10
        return 0
    fi
    LTE_RST_CHIP=$(gpiodetect | grep "${I2C_BUS}-00${I2C_ADD}" | sed "s/gpiochip//" | awk '{print $1}')
    [ $LTE_RST_CHIP ] || die "Can't read chip number of reset pin"
    LTE_RST_PIN=$(gpioinfo ${LTE_RST_CHIP} | grep "4G_RST" | cut -d ":" -f 1 | awk '{print $2}')
    [ $LTE_RST_PIN ] || die "Not found 4G reset pin"
    LTE_LED=$(gpioinfo ${LTE_RST_CHIP} | grep "4G_LED" | cut -d ":" -f 1 | awk '{print $2}')
    [ $LTE_LED ] || echo_log "ERROR: Not found 4G LED pin"
    echo_log "CHIP=$LTE_RST_CHIP, RES_PIN=$LTE_RST_PIN, LED=$LTE_LED"

    m_4g_connection=$(read_ini_file $BASE_CONFIG_INI "4G" "connection")
    if [ $? -eq 0 ];then
        if [ ${#m_4g_connection} -gt 0 ];then
            CONNEECTION_NAME=$m_4g_connection
        fi
    fi
}

function lte_led_on(){
    gpioset ${LTE_RST_CHIP} ${LTE_LED}=0
}

function lte_led_off(){
    gpioset ${LTE_RST_CHIP} ${LTE_LED}=1
}

function do_lte_rst() {
    if [ $is_ipc -ne 1 ];then
        raspi-gpio set $LTE_RST_PIN op dh
        sleep 1
        raspi-gpio set $LTE_RST_PIN op dl
    else
        gpioset ${LTE_RST_CHIP} ${LTE_RST_PIN}=0
        sleep 1
        gpioset ${LTE_RST_CHIP} ${LTE_RST_PIN}=1
    fi
    # it takes a while to load the device after reset
    sleep 5

    until [ -e /sys/class/net/${IFNAME} ]
    do
        sleep 1
    done
}

function do_connect() {
    is_networkmanager
    if [ $? -eq 0 ]; then
        echo_log "Down connection ..."
        nmcli device disconnect ${DEV} > /dev/null

        echo_log "Reset lte module ..."
        do_lte_rst
        while true
        do
            sleep 1
            nmcli device show ${DEV} >> /dev/null 2>&1
            if [ $? -eq 0 ];then
                break
            fi
        done

        nmcli connection show | grep $CONNEECTION_NAME  > /dev/null
        if [ $? -ne 0 ];then
            # create connection
            m_apn=$(read_ini_file $BASE_CONFIG_INI "4G" "apn")
            t_apn='nmcli connection add type gsm autoconnect yes con-name '$CONNEECTION_NAME' ifname '$DEV
            echo $m_apn | grep ',' > /dev/null
            if [ $? -eq 0 ];then
                m_array=(${m_apn//,/ })
                if [ ${#m_array[@]} -gt 0 ];then
                    t_apn=$t_apn' gsm.apn '$m_array[0]
                fi
                if [ ${#m_array[@]} -gt 1 ];then
                    t_apn=$t_apn' gsm.username "'$m_array[1]'"'
                fi
                if [ ${#m_array[@]} -gt 2 ];then
                    t_apn=$t_apn' gsm.password "'$m_array[2]'"'
                fi
            else
                if [ ${#m_apn} -gt 0 ];then
                    t_apn=$t_apn' gsm.apn '$m_apn
                fi
            fi

            echo_log "$t_apn"
            $t_apn
        fi

        echo_log "Try to reconnect ..."
        # nmcli device connect ${DEV}
        systemctl restart NetworkManager.service
    else
        killall -q quectel-CM
        echo_log "Reset lte module ..."
        do_lte_rst
        printf "Try to connect ...\n"
        if [ ! -d "/var/log/ed-ec20-qmi/" ];then
            mkdir -p /var/log/ed-ec20-qmi/
        fi
        $BSP_HOME_PATH/quectel-CM -4 -f /var/log/ed-ec20-qmi/quectel-CM.log &
    fi
}

function check_device_type(){
    cat /sys/kernel/debug/gpio | grep "4G_RST" > /dev/null
    [ $? -eq 0 ] || sleep 5
    cat /sys/kernel/debug/gpio | grep "4G_RST" > /dev/null
    [ $? -eq 0 ] || is_ipc=0

    if [ $is_ipc -eq 0 ];then
        echo_log "This device is CM4 Sensing or CM4 Industrial"
    else
        echo_log "This device is IPCxxxx or HMIxxxx"

        while true;do
            gpiodetect | grep "${I2C_BUS}-00${I2C_ADD}" > /dev/null
            if [ $? -eq 0 ];then
                break
            fi
            sleep 1
        done
    fi

    
}

function check_network_running(){
    declare -A m_servers
    m_whiletlist=$(read_ini_file $BASE_CONFIG_INI "4G" "whilelist")
    # echo_log $m_whiletlist
    if [ $? -eq 0 ];then
        m_array=(${m_whiletlist//;/ })
        for v in ${m_array[@]};
        do
            if [ ${#v} -ne 0 ];then
                m_servers[${#m_servers[@]}]=$v
            fi
        done
    else
        m_servers[0]=$PING_IP_1
        m_servers[1]=$PING_IP_2
    fi
    
    for i in {1..4};
    do
        for t_ip in ${m_servers[@]}
        do
            ping -I ${IFNAME} -c 1 -s 0 ${t_ip} > /dev/null
            if [ $? -eq 0 ]; then
                return 0;
            fi
        done
        sleep 1
    done
    return 1;
}

function do_start(){
    m_auto=$(read_ini_file $BASE_CONFIG_INI "4G" "auto")
    echo "auto start: "$m_auto
    if [ $? -eq 0 ]; then
        if [ "$m_auto" = "1" ]; then
            check_device_type
            do_lte_init_rst_pin
            [ $is_ipc -ne 1 ] || lte_led_off
            index=10;
            while true; do
                check_network_running
                if [ $? -ne 0 ];then
                    [ $is_ipc -ne 1 ] || lte_led_off
                    if [ $index -lt 10 ];then
                        index=`expr $index + 1`
                    else
                        do_connect
                        index=0
                    fi
                else
                    [ $is_ipc -ne 1 ] || lte_led_on
                    index=0
                fi
                sleep 10
            done
        fi
    fi
}

do_start