#!/bin/bash

LTE_RST_CHIP=
LTE_RST_PIN=
LTE_LED=
DEV=cdc-wdm0
IFNAME=wwan0
I2C_BUS=10
I2C_ADD=20
is_ipc=1

function echo_log(){
    local time_now=$(date +%Y/%m/%d,%H:%M:%S)
    echo "[$time_now] $1"
}

function die(){
    echo_log "ERROR: $1"
    exit 1  
}

function do_lte_init_rst_pin() {
    if [ $is_ipc -ne 1 ];then
        LTE_RST_PIN=10
        return 0
    fi
    LTE_RST_CHIP=$(gpiodetect | grep "${I2C_BUS}-00${I2C_ADD}" | sed "s/gpiochip//" | awk '{print $1}')
    [ $LTE_RST_CHIP ] || die "Can't read chip number of reset pin"
    LTE_RST_PIN=$(gpioinfo ${LTE_RST_CHIP} | grep "4G_RST" | cut -d ":" -f 1 | awk '{print $2}')
    [ $LTE_RST_PIN ] || die "Not found 4G reset pin"
    LTE_LED=$(gpioinfo ${LTE_RST_CHIP} | grep "4G_LED" | cut -d ":" -f 1 | awk '{print $2}')
    [ $LTE_LED ] || echo_log "ERROR: Not found 4G LED pin"
    echo_log "CHIP=$LTE_RST_CHIP, RES_PIN=$LTE_RST_PIN, LED=$LTE_LED"
}

function do_lte_rst() {
    if [ $is_ipc -ne 1 ];then
        raspi-gpio set $LTE_RST_PIN op dh
        sleep 1
        raspi-gpio set $LTE_RST_PIN op dl
    else
        gpioset ${LTE_RST_CHIP} ${LTE_RST_PIN}=0
        sleep 1
        gpioset ${LTE_RST_CHIP} ${LTE_RST_PIN}=1
    fi
    # it takes a while to load the device after reset
    sleep 5

    until [ -e /sys/class/net/${IFNAME} ]
    do
        sleep 1
    done
}

function check_device_type(){
    cat /sys/kernel/debug/gpio | grep "4G_RST" > /dev/null
    [ $? -eq 0 ] || sleep 5
    cat /sys/kernel/debug/gpio | grep "4G_RST" > /dev/null
    [ $? -eq 0 ] || is_ipc=0

    if [ $is_ipc -eq 0 ];then
        echo_log "This device is CM4 Sensing or CM4 Industrial"
    else
        echo_log "This device is IPCxxxx or HMIxxxx"

        while true;do
            gpiodetect | grep "${I2C_BUS}-00${I2C_ADD}" > /dev/null
            if [ $? -eq 0 ];then
                break
            fi
            sleep 1
        done
    fi

    
}

function do_start(){
    check_device_type
    do_lte_init_rst_pin
    do_lte_rst
}

do_start